package cn.wisenergy.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.wisenergy.common.utils.DateUtil;
import cn.wisenergy.common.utils.ExcelUtils;
import cn.wisenergy.common.utils.exception.BASE_RESP_CODE_ENUM;
import cn.wisenergy.common.utils.exception.BaseCustomException;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.dto.*;
import cn.wisenergy.model.enums.*;
import cn.wisenergy.service.StatisticsService;
import cn.wisenergy.service.WorkProjectService;
import cn.wisenergy.service.WorkUserService;
import cn.wisenergy.service.utils.UserRoleLevelUtils;
import com.alibaba.excel.util.StringUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @description:
 * @author: nh
 * @create: 2021-01-19 17:19
 **/

@Service
@Slf4j
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    private WorkUserService workUserService;

    @Autowired
    private WorkTimeOrderMapper workTimeOrderMapper;

    @Autowired
    private WorkUserDeptMapper workUserDeptMapper;

    @Autowired
    private WorkProjectService workProjectService;

    @Autowired
    private WorkHolidayMapper workHolidayMapper;

    @Autowired
    private WorkUserMapper workUserMapper;

    @Autowired
    private WorkTypeMapper workTypeMapper;

    @Autowired
    private WorkProjectTimeCostMapper workProjectTimeCostMapper;

    @Autowired
    private WorkProjectMapper workProjectMapper;

    @Autowired
    private WorkDeptMapper workDeptMapper;




    @Autowired
    private WorkUserProjectMapper workUserProjectMapper;

    private final static Integer EXCEL_WIDTH = 1000 * 6;

    private final static Integer DEFAULT_DEPT_ID = 1;

    @Override
    public PageInfo getDayWorkTimeOfMonth(GetMonthlyCollectParam param) {
        log.info("StatisticsServiceImpl[]getMonthlyCollect[]input.param" + param);
        if (param == null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.INPUT_PARAM_IS_NULL);
        }
        // 用户权限名
        List<String> userAuthority = this.getUserAuthority(param.getUserId());

        if (CollectionUtil.isEmpty(userAuthority) || !userAuthority.contains(LevelEnum.TIMEEXPORT.getLevelName())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.THE_USER_NOT_MANAGER_PLASE_MANAGER_LOGIN);
        }
        //默认时间为当月
        Integer year = param.getYear();
        Integer month = param.getMonth();
        if (StringUtils.isEmpty(param.getYear()) || StringUtils.isEmpty(param.getMonth())) {
            Date now = new Date();
            year = DateUtil.getYear(now);
            month = DateUtil.getMonth(now) + 1;
        }
        //默认部门主键为1
        Integer deptId = param.getDeptId();
        if (param.getDeptId() == null) {
            deptId = DEFAULT_DEPT_ID;
        }
        //默认当前页码为1，每页数量为10
        if (param.getCurrentPage() == null) {
            param.setCurrentPage(1);
        }
        if (param.getSize() == null) {
            param.setSize(10);
        }

        // 分页获取部门下所有员工主键
        PageHelper.startPage(param.getCurrentPage(), param.getSize());
        List<Integer> userIds = workUserDeptMapper.listByDeptIdAndStatus(deptId);
        PageInfo pageInfo = new PageInfo(userIds);
        // 获取用户姓名、部门名称
        List<StatisticsTableDto> statisticsTableDtos = workUserMapper.getStatisticsTableDtos(userIds);
        // 获取用户当月每天工时信息
        List<DayWorkTimeAndType> dayWorkTimeAndTypes = workTimeOrderMapper.listByDateAndUserId(year, month, userIds);

        // 将工单集合与对应用户信息封装在一起
        Iterator<StatisticsTableDto> statisticsTableDtoIterator = statisticsTableDtos.iterator();
        while (statisticsTableDtoIterator.hasNext()) {
            StatisticsTableDto statisticsTableDto = statisticsTableDtoIterator.next();
            // 存放一位用户一个月中每天的工时信息
            ArrayList<DayWorkTimeAndType> dayWorkTimeAndTypes1 = new ArrayList<>();
            // 查询出的工时信息遍历
            Iterator<DayWorkTimeAndType> workTimeAndTypeIterator = dayWorkTimeAndTypes.iterator();
            while (workTimeAndTypeIterator.hasNext()) {
                DayWorkTimeAndType dayWorkTimeAndType = workTimeAndTypeIterator.next();
                if (dayWorkTimeAndType.getUserId().equals(statisticsTableDto.getUserId())) {
                    dayWorkTimeAndTypes1.add(dayWorkTimeAndType);
                }
            }
            statisticsTableDto.setDayWorkTimeAndTypeList(dayWorkTimeAndTypes1);
        }
        pageInfo.setList(statisticsTableDtos);
        return pageInfo;
    }

    @Override
    public Map<String, Object> getMonthlyStatistics(Integer userId) {
        log.info("WorkTimeOrderServiceImpl[]getMonthlyStatistics[]input.param" + userId);
        WorkUser user = workUserService.getById(userId);
        // 用户权限
        List<String> userAuthority = this.getUserAuthority(userId);
        // 用户权限与权限对应级别
        List<Integer> role = UserRoleLevelUtils.getRole(userId);
        List<Integer> list = UserRoleLevelUtils.getlevelIds(role);
        List<WorkLevel> workLevels = UserRoleLevelUtils.getlevelByIds(list);
        List<Integer> rank = this.getUserRank(workLevels, LevelEnum.EXAMINE.getLevelName());
        // 本月第一天
        Date firstDayOfMonth = DateUtil.getCurrentFirstDayOfMonth();
        // 返回数据对象
        Map<String, Object> objects = new HashMap<>();
        // 存放查询条件
        Map<String, Object> map = new HashMap<>();
        // 用户有审批权限
        if (CollectionUtil.isNotEmpty(userAuthority) && userAuthority.contains(LevelEnum.EXAMINE.getLevelName())) {
            //以审批人、审核状态、开始时间查询本月审批数量
            map.put("reviewerId", userId);
            map.put("startModifyTime", firstDayOfMonth);
            Integer completedCount = workTimeOrderMapper.getCountByCondition(map);

            //本月驳回未重报数量
            map.put("status", WorkOrderStatus.TURN_DOWN.getCode());
            Integer rejectAndNotResubmit = workTimeOrderMapper.getCountByCondition(map);

            // 移除查询审批人和状态条件
            map.remove("reviewerId");
            map.remove("status");
            //待审核包括：新提交和重新提交的
            List<Integer> statusArray = new ArrayList<>();
            statusArray.add(WorkOrderStatus.WAIT_AUDIT.getCode());
            statusArray.add(WorkOrderStatus.RE_SUBMIT.getCode());
            //本月未审批数量查询
            map.put("statusArray", statusArray);
            // 查询用户负责的项目和商机id
            List<Integer> projectIdList = workProjectMapper.getProjectIdByManager(userId);
            Integer notCompletedCount1 = 0;
            if (CollectionUtil.isNotEmpty(projectIdList)) {
                map.put("projectIdList", projectIdList);
                // 项目和商机的未审批的数
                notCompletedCount1 = workTimeOrderMapper.getCountByCondition(map);
            }

            // 查询用户负责审批的除项目合商机的其余类型
            Integer notCompletedCount2 = 0;
            if (rank.contains(LevelRankEnum.DEPARTMENT_LEVEL.getRank())) {
                map.remove("projectIdList");
                List<WorkType> allType = workTypeMapper.getAll();
                List<Integer> type = new ArrayList<>();
                for (int i = 4; i < allType.size(); i++) {
                    type.add(allType.get(i).getId());
                }
                map.put("typeList", type);
                map.put("deptId", user.getDeptId());
                // 其余类型为审批的条数
                notCompletedCount2 = workTimeOrderMapper.getCountByCondition(map);
            }
            //所有查询结果封装进对象
            ApprovalStatistics approvalStatistics = new ApprovalStatistics();
            approvalStatistics.setApprovalCount(completedCount);
            approvalStatistics.setRejectNotSubmit(rejectAndNotResubmit);
            approvalStatistics.setNotApprovalCount(notCompletedCount1 + notCompletedCount2);
            objects.put("manager", approvalStatistics);
        }

        // 填报情况
        // 本月应填报日期
        List<Date> currentMonthWorkDays = this.currentMonthWorkDays(firstDayOfMonth, true);
        // 本月已填报日期
        List<Date> currentMonthFilledDays = this.currentMonthFilledDays(userId, firstDayOfMonth);
        // 本月未填报日期
        currentMonthWorkDays.removeAll(currentMonthFilledDays);
        int notFilledCount = currentMonthWorkDays.size();
        //本月被现存驳回数
        map.clear();
        map.put("status", StatusEnum.REJECTED.getCode());
        map.put("userId", userId);
        map.put("startWorkTime", firstDayOfMonth);
        Integer countByCondition = workTimeOrderMapper.getCountByCondition(map);

        List<MonthlyWorkingHoursStatistics> monthlyWorkingHoursStatistics = workTimeOrderMapper.statisticsByProjectType(userId, firstDayOfMonth);
        Integer[] status = {StatusEnum.APPROVED.getCode(), StatusEnum.AUTOMATIC_AUDIT.getCode()};
        // 已审批的类型和统计条数
        List<MonthlyWorkingHoursStatistics> monthlyWorkingHoursStatistics1 = workTimeOrderMapper.selectCountByTypeAndStatus(userId, firstDayOfMonth, status);
        for (MonthlyWorkingHoursStatistics workingHoursStatistics : monthlyWorkingHoursStatistics1) {
            for (MonthlyWorkingHoursStatistics monthlyWorkingHoursStatistic : monthlyWorkingHoursStatistics) {
                if (workingHoursStatistics.getProjectType().equals(monthlyWorkingHoursStatistic.getProjectType())) {
                    monthlyWorkingHoursStatistic.setApproved(workingHoursStatistics.getApproved());
                }
            }
        }

        List<Integer> types = monthlyWorkingHoursStatistics.stream().map(MonthlyWorkingHoursStatistics::getProjectType).collect(Collectors.toList());
        // 如果没填写填报类型其中某项，造对象传前端
        List<WorkType> workTypes = workTypeMapper.getAll();
        if (monthlyWorkingHoursStatistics.size() < workTypes.size()) {
            for (WorkType workType : workTypes) {
                if (!types.contains(workType.getId())) {
                    MonthlyWorkingHoursStatistics workingHoursStatistics = new MonthlyWorkingHoursStatistics();
                    workingHoursStatistics.setTypeName(workType.getName());
                    workingHoursStatistics.setProjectType(workType.getId());
                    workingHoursStatistics.setCount(0);
                    workingHoursStatistics.setWorkTime(0);
                    monthlyWorkingHoursStatistics.add(workingHoursStatistics);
                }
            }
        }
        // 调整类型顺序
        Collections.sort(monthlyWorkingHoursStatistics);

        // 赋值未填报天数、被驳回未重报天
        for (MonthlyWorkingHoursStatistics monthlyWorkingHoursStatistic : monthlyWorkingHoursStatistics) {
            if (monthlyWorkingHoursStatistic.getApproved() == null) {
                monthlyWorkingHoursStatistic.setApproved(0);
            }
        }
        objects.put("notFilledCount", notFilledCount);
        objects.put("countByCondition", countByCondition);
        objects.put("user", monthlyWorkingHoursStatistics);
        return objects;
    }

    @Override
    public List<WorkTimeAndCostCollect> getCurrentMonthWorkTimeCollect(Integer userId, String startTime) {
        log.info("StatisticsServiceImpl[]getCurrentMonthWorkTimeCollect[]input.param" + userId + startTime);
        if (userId == null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.INPUT_PARAM_IS_NULL);
        }
        // 用户角色
        List<Integer> role = UserRoleLevelUtils.getRole(userId);
        // 用户权限
        List<String> userAuthority = this.getUserAuthority(userId);
        // 没有审批权限
        if (CollectionUtil.isEmpty(userAuthority) || !userAuthority.contains(LevelEnum.EXAMINE.getLevelName())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.THE_USER_NOT_MANAGER_PLASE_MANAGER_LOGIN);
        }
        WorkUser user = workUserService.getById(userId);

        // 时间不为空，赋值当月第一天
        if (StringUtil.isNotEmpty(startTime)) {
            Date currentFirstDayOfMonth = DateUtil.getCurrentFirstDayOfMonth();
            startTime = DateUtil.convertDateToYMDStr(currentFirstDayOfMonth);
        }
        // 获取角色
        List<Integer> list = UserRoleLevelUtils.getlevelIds(role);
        // 获取角色对应的权限
        List<WorkLevel> workLevels = UserRoleLevelUtils.getlevelByIds(list);
        // 用户该权限的级别
        List<Integer> rank = this.getUserRank(workLevels, LevelEnum.EXAMINE.getLevelName());

        // 获取所有的类型
        List<WorkType> workTypes = workTypeMapper.getAll();
        // 存储查询的类型
        List<Integer> types = new ArrayList<>();
        // 只存储项目和商机之外的类型
        for (WorkType workType : workTypes) {
            if (workType.getId() != 1 && workType.getId() != 2) {
                types.add(workType.getId());
            }
        }

        List<WorkTimeAndCostCollect> workTimeAndCostCollects = null;
        // 中心级审批权限
        if (rank.contains(LevelRankEnum.CENTRAL_LEVEL.getRank())) {
            // 获取用户所在中心
            WorkDept userDept = workDeptMapper.getById(user.getDeptId());
            // 获取中心下的所有项目和商机工单统计
            Map<String, Object> map = new HashMap<>();
            map.put("firstDayOfMonth", startTime);
            map.put("centreId", userDept.getCentreId());
            workTimeAndCostCollects = workProjectTimeCostMapper.selectList(map);

            // 除项目和项目的类型
            map.put("types", types);
            // 获取这些类型的所有工单
            List<WorkTimeAndCostCollect> workTimeAndCostCollect = workTimeOrderMapper.getWorkTimeAndCostCollect(map);
            workTimeAndCostCollects.addAll(workTimeAndCostCollect);

        } else if (rank.contains(LevelRankEnum.DEPARTMENT_LEVEL.getRank())) { // 如果用户的审批权限的级别最高为部门级别
            // 获取部门下的所有项目和商机的工时统计
            Map<String, Object> map = new HashMap<>();
            map.put("deptId", user.getDeptId());
            map.put("firstDayOfMonth", startTime);
            workTimeAndCostCollects = workProjectTimeCostMapper.selectList(map);

            // 除项目和项目的类型
            map.put("types", types);
            // 获取这些类型的所有工单
            List<WorkTimeAndCostCollect> workTimeAndCostCollect = workTimeOrderMapper.getWorkTimeAndCostCollect(map);
            workTimeAndCostCollects.addAll(workTimeAndCostCollect);
        } else if (rank.contains(LevelRankEnum.PROJECT_LEVEL.getRank())) { // 如果用户的审批权限的级别最高为项目/商机级别,与其他两种级别不同的是只汇总项目和商机类型的工单
            // 获取部门下的所有项目和商机的工时统计
            Map<String, Object> map = new HashMap<>();
            // 获取用户负责项目和商机id
            List<Integer> projectIdByManager = workProjectMapper.getProjectIdByManager(userId);
            map.put("projectIds", projectIdByManager);
            map.put("firstDayOfMonth", startTime);
            workTimeAndCostCollects = workProjectTimeCostMapper.selectList(map);
        }

        // 获取查询结果中的类型
        List<Integer> typeList = workTimeAndCostCollects.stream().map(WorkTimeAndCostCollect::getType).collect(Collectors.toList());
        // 如果查询结果不包含查询条件中的类型，造对象传前端
        for (WorkType workType : workTypes) {
            if (!typeList.contains(workType.getId())) {
                // 权限级别为项目级别的用户只返回项目和商机类型的汇总
                if (rank.contains(LevelRankEnum.PROJECT_LEVEL.getRank()) && rank.size() == 1 && types.contains(workType.getId())) {
                    continue;
                }
                WorkTimeAndCostCollect workTimeAndCostCollect1 = new WorkTimeAndCostCollect();
                workTimeAndCostCollect1.setType(workType.getId());
                workTimeAndCostCollect1.setTypeName(workType.getName());
                workTimeAndCostCollects.add(workTimeAndCostCollect1);
            }
        }

        // 给类型赋值类型名称
        for (WorkTimeAndCostCollect timeAndCostCollect : workTimeAndCostCollects) {
            for (WorkType workType : workTypes) {
                if (timeAndCostCollect.getType().equals(workType.getId())) {
                    timeAndCostCollect.setTypeName(workType.getName());
                }
            }
        }
        // 类型总工时
        for (WorkTimeAndCostCollect workTimeAndCostCollect1 : workTimeAndCostCollects) {
            // 类型的总工时
            BigDecimal typeTotalTime = new BigDecimal("0.000");
            List<DeptProjectWorkTimeAndCost> deptProjectWorkTimeAndCosts = workTimeAndCostCollect1.getDeptProjectWorkTimeAndCosts();
            if (CollectionUtil.isNotEmpty(deptProjectWorkTimeAndCosts)) {
                for (DeptProjectWorkTimeAndCost deptProjectWorkTimeAndCost : deptProjectWorkTimeAndCosts) {
                    List<ProjectWorkTimeAndCostStatistics> projectWorkTimeAndCostStatistics = deptProjectWorkTimeAndCost.getProjectWorkTimeAndCostStatistics();
                    for (ProjectWorkTimeAndCostStatistics projectWorkTimeAndCostStatistic : projectWorkTimeAndCostStatistics) {
                        typeTotalTime = typeTotalTime.add(projectWorkTimeAndCostStatistic.getTotalTime());
                    }
                }
            }
            workTimeAndCostCollect1.setWorkTime(typeTotalTime);
        }

        // 排序
        Collections.sort(workTimeAndCostCollects);
        return workTimeAndCostCollects;
    }

    @Override
    public HSSFWorkbook exportWorkTimeExcel(GetMonthlyCollectParam param) {
        log.info("StatisticsServiceImpl[]exportWorkTimeExcel[]input.param{}" + param);
        PageInfo dayWorkTimeOfMonth = this.getDayWorkTimeOfMonth(param);
        Date date = DateUtil.convertStrToDate(param.getYear() + "-" + param.getMonth(), "yyyy-MM");
        // 传参的时间当月总天数
        int dayCount = DateUtil.getMonthDaysArray(date).length;
        // 需要导出的信息
        List<StatisticsTableDto> list = dayWorkTimeOfMonth.getList();
        // 创建工作簿
        HSSFWorkbook sheets = new HSSFWorkbook();
        // 创建表
        HSSFSheet sheet = sheets.createSheet("Sheet1");
        // 普通样式对象
        HSSFCellStyle cellStyle = this.createOrdinaryCellStyle(sheets);
        cellStyle.setFillForegroundColor(HSSFColor.HSSFColorPredefined.WHITE.getIndex());

        // 节假日样式，底色为绿色,其余样式与普通相同
        HSSFCellStyle cellStyle1 = this.createOrdinaryCellStyle(sheets);
        cellStyle1.setFillForegroundColor(HSSFColor.HSSFColorPredefined.LIGHT_GREEN.getIndex());

        // 当日工时小于8，底色为红色，其余样式与普通相同
        HSSFCellStyle cellStyle2 = this.createOrdinaryCellStyle(sheets);
        cellStyle2.setFillForegroundColor(HSSFColor.HSSFColorPredefined.RED.getIndex());

        // 第一列列宽
        sheet.setColumnWidth(1, EXCEL_WIDTH);
        // 获取本月应上班日期集合
        List<Date> dates = this.currentMonthWorkDays(date, false);
        List<Integer> days = new ArrayList<>();
        for (Date date1 : dates) {
            days.add(DateUtil.getDay(date1));
        }
        // 创建一行
        HSSFRow row1 = sheet.createRow(0);
        // 给第一行赋值
        row1.createCell(0).setCellValue("序号");
        row1.createCell(1).setCellValue("部门名称");
        row1.createCell(2).setCellValue("姓名");
        for (int i = 1; i <= dayCount; i++) {
            HSSFCell cell = row1.createCell(2 + i);
            cell.setCellValue(i + "日");
            // 如果为节假日
            if (!days.contains(i)) {
                cell.setCellStyle(cellStyle1);
            } else {
                cell.setCellStyle(cellStyle);
            }
        }
        Iterator<StatisticsTableDto> iterator = list.iterator();
        int i = 1;
        while (iterator.hasNext()) {
            StatisticsTableDto next = iterator.next();
            // 创建对应的行
            HSSFRow row_i = sheet.createRow(i);
            // 序号
            row_i.createCell(0).setCellValue(i);
            // 部门名称
            row_i.createCell(1).setCellValue(next.getDeptName());
            // 姓名
            row_i.createCell(2).setCellValue(next.getUserName());
            // 日期与工时集合
            List<DayWorkTimeAndType> timeAndTypes = next.getDayWorkTimeAndTypeList();
            // 在excel中对应日期单元格添加工时
            for (DayWorkTimeAndType timeAndType : timeAndTypes) {
                HSSFCell cell = row_i.getCell(timeAndType.getDay() + 2);
                String cellValue = ExcelUtils.getCellValue(cell, true);
                // 单元格为空则直接写入，否则追加
                if (!StringUtils.isEmpty(cellValue)) {
                    cell.setCellValue(timeAndType.getType() + timeAndType.getWorkTime().toString() + "\n" + cellValue);
                } else {
                    HSSFCell emptyCell = row_i.createCell(timeAndType.getDay() + 2);
                    emptyCell.setCellValue(timeAndType.getType() + timeAndType.getWorkTime().toString());
                }
            }
            // 换行
            i++;
        }
        // 给节假日单元格设置单元格格式
        for (int j = 0; j < list.size(); j++) {
            HSSFRow row = sheet.getRow(j + 1);
            for (int r = 0; r < dayCount; r++) {
                HSSFCell cell = null;
                if (row.getCell(r + 3) == null) {
                    cell = row.createCell(r + 3);
                } else {
                    cell = row.getCell(r + 3);
                }
                // 节假日单元格标绿色
                if (!days.contains(r + 1)) {
                    cell.setCellStyle(cellStyle1);
                } else {
                    cell.setCellStyle(cellStyle);
                }
                // 获取单元格中的工时，当日总工时小于8,则单元格底色为红色
//                String cellValue = ExcelUtils.getCellValue(cell, true);
//                if (StringUtil.isNotEmpty(cellValue)) {
//                    Integer numberOfString = this.getNumberOfString(cellValue);
//                    if (numberOfString != null && numberOfString < 8) {
//                        cell.setCellStyle(cellStyle2);
//                    }
//                }
            }
        }
        return sheets;
    }

    // 将字符串中的1到9的数字相加
    private Integer getNumberOfString(String value) {
        value = value.trim();
        if (StringUtil.isEmpty(value)) {
            return null;
        }
        Integer sum = 0;
        for (int i = 0; i < value.length(); i++) {
            if (value.charAt(i) >= 48 && value.charAt(i) <= 57) {
                sum += value.charAt(i) - 48;
            }
        }
        return sum;
    }

    @Override
    public List<UserWorkTimeStatisticsByProject> getUserWorkTimeStatisticsReport(Integer userId, Integer deptId, String startDate, String endDate, Integer isConclusion) {
        log.info("StatisticsServiceImpl[]getUserWorkTimeStatisticsReport[]" + userId + deptId + startDate + endDate + isConclusion);
        List<Integer> role = UserRoleLevelUtils.getRole(userId);
        // 判断是否有该权限
        if (!UserRoleLevelUtils.isManagerLevel(userId, LevelEnum.USER_STATISTICS.getLevelName())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.NO_AUTHORITY);
        }
        // 获取用户信息
        WorkUser user = workUserService.getById(userId);
        // 获取用户权限的级别
        List<Integer> list = UserRoleLevelUtils.getlevelIds(role);
        List<WorkLevel> workLevels = UserRoleLevelUtils.getlevelByIds(list);
        List<Integer> rank = this.getUserRank(workLevels, LevelEnum.USER_STATISTICS.getLevelName());
        // 不是中心级不能选择部门
        if (!rank.contains(LevelRankEnum.CENTRAL_LEVEL.getRank()) && deptId != null) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.NO_AUTHORITY);
        }
        // 中心级权限默认查询部门
        if (rank.contains(LevelRankEnum.CENTRAL_LEVEL.getRank()) && deptId == null) {
            deptId = DEFAULT_DEPT_ID;
        }

        // 默认统计截至时间为当天
        if (StringUtil.isEmpty(endDate)) {
            Date date = new Date();
            endDate = DateUtil.convertDateToYMDStr(date);
        }
        Map<String, Object> map = new HashMap<>();
        map.put("isConclusion", isConclusion);
        List<WorkProject> userManageProjects = null;
        // 当前权限最高为中心级别权限
        if (rank.contains(LevelRankEnum.CENTRAL_LEVEL.getRank())) {
            // 查询部门下所有项目，部门主键为空时是默认部门
            map.put("deptId", deptId);
            userManageProjects = workProjectMapper.getListByCriteria(map);
        } else if (rank.contains(LevelRankEnum.DEPARTMENT_LEVEL.getRank())) {
            // 查询部门下所有项目，部门主键是用户的部门主键,当前权限最高级别为部门级别
            map.put("deptId", user.getDeptId());
            userManageProjects = workProjectMapper.getListByCriteria(map);
        } else {
            map.put("managerId", user.getId());
            userManageProjects = workProjectMapper.getListByCriteria(map);
        }

        // 管理项目为空则提前结束
        if (CollectionUtil.isEmpty(userManageProjects)) {
            return null;
        }
        // 提取项目主键
        List<Integer> projectIds = userManageProjects.stream().map(WorkProject::getId).collect(Collectors.toList());

        // 查询各个项目正常工时和加班工时统计总计
        List<UserWorkTimeStatisticsByProject> userProjectWorkTimeStatistics = workTimeOrderMapper.getUserProjectWorkTimeStatistics(startDate, endDate, projectIds, false);

        // 查询除开项目于、商机、请假、调休这几种类型的工单
        List<WorkType> types = workTypeMapper.getAll();
        List<Integer> queryType = new ArrayList<>();
        for (WorkType type : types) {
            if (type.getId() > 4) {
                queryType.add(type.getId());
            }
        }
        // 查询各个项目下所有人的加班工时和平常工时
        List<UserWorkTimeStatisticsByProject> userProjectWorkTimeStatistics1 = workTimeOrderMapper.getUserProjectWorkTimeStatistics(startDate, endDate, projectIds, true);

        // 部门级别以上的用户可以查询所管理项目的成员所填的其他类型的工单
        List<Integer> userIds = workUserProjectMapper.selectUserIdByProjectIds(projectIds);
        if (rank.contains(LevelRankEnum.DEPARTMENT_LEVEL.getRank()) || rank.contains(LevelRankEnum.CENTRAL_LEVEL.getRank())) {
            List<UserWorkTimeStatisticsByProject> userWorkTimeStatisticsByProjectList = workTimeOrderMapper.selectListByType(startDate, endDate, queryType, userIds, false);
            if (CollectionUtil.isNotEmpty(userWorkTimeStatisticsByProjectList)) {
                userProjectWorkTimeStatistics.get(0).getProjectWorkTimeAndType().addAll(userWorkTimeStatisticsByProjectList.get(0).getProjectWorkTimeAndType());
            }
            // 查询各个项目下所有人的其他除了项目、商机、请假、调休类型的加班工时和平常工时
            List<UserWorkTimeStatisticsByProject> userWorkTimeStatisticsByProjects = workTimeOrderMapper.selectListByType(startDate, endDate, queryType, userIds, true);
            for (UserWorkTimeStatisticsByProject userWorkTimeStatisticsByProject : userWorkTimeStatisticsByProjects) {
                for (UserWorkTimeStatisticsByProject workTimeStatisticsByProject : userProjectWorkTimeStatistics1) {
                    if (userWorkTimeStatisticsByProject.getUserId() != null && userWorkTimeStatisticsByProject.getUserId().equals(workTimeStatisticsByProject.getUserId())) {
                        workTimeStatisticsByProject.getProjectWorkTimeAndType().addAll(userWorkTimeStatisticsByProject.getProjectWorkTimeAndType());
                    }
                }
            }
        }
        userProjectWorkTimeStatistics.addAll(userProjectWorkTimeStatistics1);

        // 取出单人的项目工时统计
        for (UserWorkTimeStatisticsByProject userProjectWorkTimeStatistic : userProjectWorkTimeStatistics) {
            int totalTime = 0;
            List<ProjectWorkTimeAndType> projectWorkTimeAndType1 = userProjectWorkTimeStatistic.getProjectWorkTimeAndType();
            // 遍历一个人所有项目统计，计算所有项目所有工时总和
            for (ProjectWorkTimeAndType projectWorkTimeAndType : projectWorkTimeAndType1) {
                projectWorkTimeAndType.setProjectTypeName(types.get(projectWorkTimeAndType.getProjectType() - 1).getName());
                totalTime += projectWorkTimeAndType.getWorkTime();
            }
            userProjectWorkTimeStatistic.setTotalTime(totalTime);
        }

        return userProjectWorkTimeStatistics;
    }

    @Override
    public List<ProjectStatisticsByMonth> getProjectStatistics(Integer userId, Integer deptId, Integer year, Integer projectId) {
        log.info("StatisticsServiceImpl[]getProjectStatistics[]input.param" + userId + deptId + year + projectId);
        // 获取权限
        if (!UserRoleLevelUtils.isManagerLevel(userId, LevelEnum.PROJECT_STATISTICS.getLevelName())) {
            throw new BaseCustomException(BASE_RESP_CODE_ENUM.NO_AUTHORITY);
        }

        // 默认查询时间为当前
        if (year == null) {
            year = DateUtil.getYear(new Date());
        }
        // 部门主键和项目主键不能同时为空
        if (deptId == null && projectId == null) {
            deptId = DEFAULT_DEPT_ID;
        }

        // 部门为空，查询某个项目的全周期统计，不为空，查询部门在某年的全部项目按月总计信息
        List<ProjectStatisticsByMonth> projectStatisticsByMonths = workProjectTimeCostMapper.selectListByMonth(year, deptId, projectId);
        if (CollectionUtil.isEmpty(projectStatisticsByMonths)) {
            return null;
        }
        List<ProjectStatisticsByMonth> projectStatisticsByMonths1 = workProjectTimeCostMapper.selectListStatistics(year, deptId, projectId);
        // 将查询结果封装在一起，部门主键为空，项目总计在前，否则在后
        List<ProjectStatisticsByMonth> projectStatistics = new ArrayList<>();
        if (deptId == null) {
            projectStatistics.addAll(projectStatisticsByMonths);
            projectStatistics.addAll(projectStatisticsByMonths1);
        } else {
            projectStatistics.addAll(projectStatisticsByMonths1);
            projectStatistics.addAll(projectStatisticsByMonths);
        }

        // 部门主键不为空时，日期格式只有某月。为空时，日期格式为某年统计的第一个月为某年某月，其余为某月
        this.conversionDateFormat(projectStatistics, deptId);

        // 计算项目工时和成本合计
        for (ProjectStatisticsByMonth projectStatisticsByMonth : projectStatisticsByMonths1) {
            // 一条统计的
            List<ProjectStatisticsByMonth.StatisticsDateAndWorkTime> statisticsDateAndWorkTimes = projectStatisticsByMonth.getStatisticsDateAndWorkTimes();
            // 项目合计对象
            ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime1 = new ProjectStatisticsByMonth.StatisticsDateAndWorkTime();
            statisticsDateAndWorkTime1.setDate("项目合计");
            BigDecimal totalTime = new BigDecimal("0.0000");
            BigDecimal cost = new BigDecimal("0.0000");
            // 统计项目工时和成本
            for (ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime : statisticsDateAndWorkTimes) {
                totalTime = totalTime.add(statisticsDateAndWorkTime.getTotalTime());
                cost = cost.add(statisticsDateAndWorkTime.getCost());
            }
            // 项目合计对象赋值
            statisticsDateAndWorkTime1.setTotalTime(totalTime);
            statisticsDateAndWorkTime1.setCost(cost);
            statisticsDateAndWorkTimes.add(statisticsDateAndWorkTime1);
        }

        return projectStatistics;
    }

    //获取一个月应上班日期，如果是当月，时间截至到当天
    public List<Date> currentMonthWorkDays(Date date, boolean toToday) {
        log.info("StatisticsServiceImpl[]currentMonthWorkDays[]input.method");
        // 时间为本月,起始时间为参数时间当月第一天，截至时间为当天。否则，截至时间为当月最后一天
        Date startDate = DateUtil.getFirstDayOfMonth(date);
        Date endDate = null;
        // 当前时间
        Date now = new Date();
        if (DateUtil.getYear(date) == DateUtil.getYear(now) && DateUtil.getMonth(date) == DateUtil.getMonth(now) && toToday) {
            endDate = DateUtil.getStartOfDay(now);
        } else {
            endDate = DateUtil.getLastDayOfMonth(date);
        }
        // 当月月第一天到截止时间的日期集合
        List<Date> workDays = DateUtil.getDatesBetweenTwoDate(startDate, endDate);

        //除去周末
        Iterator<Date> iterator = workDays.iterator();
        while (iterator.hasNext()) {
            Date next = iterator.next();
            int week = DateUtil.getWeek(next);
            // week为7是星期六，1是星期天
            if (week == 7 || week == 1) {
                iterator.remove();
            }
        }
        //获取当月的节日和周末上班日期
        List<WorkHoliday> specialDates = workHolidayMapper.getByDate(startDate, endDate);
        if (CollectionUtil.isEmpty(specialDates)) {
            return workDays;
        }
        for (WorkHoliday specialDate : specialDates) {
            List<Date> isWorkDays = DateUtil.getDatesBetweenTwoDate(specialDate.getDayStart(), specialDate.getDayEnd());
            //获取的是工作日
            if (specialDate.getDayType().equals(HolidayTypeEnum.WORKING_DAY.getCode())) {
                //在本月应上班集合中加上这天，一般都是放假前后的周末调整上班
                for (Date isWorkDay : isWorkDays) {
                    if (isWorkDay.compareTo(endDate) != 1 && isWorkDay.compareTo(startDate) != -1 && !workDays.contains(isWorkDay)) {
                        workDays.add(isWorkDay);
                    }
                }
            } else {
                //否则就除去假日
                for (Date isWorkDay : isWorkDays) {
                    if (isWorkDay.compareTo(endDate) != 1 && isWorkDay.compareTo(startDate) != -1 && !workDays.contains(isWorkDay)) {
                        workDays.remove(isWorkDay);
                    }
                }
            }
        }
        return workDays;
    }

    //获取用户某月已填报的日期,如果一天中只有被驳回则不为已填报
    public List<Date> currentMonthFilledDays(Integer userId, Date date) {
        log.info("StatisticsServiceImpl[]currentMonthFilledDays[]input.method");
        List<Date> currentMonthFilledDays = workTimeOrderMapper.getDaysByDateAndStatus(userId, date);
        return currentMonthFilledDays;
    }

    // 设置单元格基本样式
    HSSFCellStyle createOrdinaryCellStyle(HSSFWorkbook sheets) {
        HSSFCellStyle cellStyle = sheets.createCellStyle();
        // 设置全填充
        cellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        // 垂直、水平中心对齐
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        cellStyle.setBorderBottom(BorderStyle.THIN);
        //设置下边框的边框线颜色（颜色和上述的颜色对照表是一样的）
        cellStyle.setBottomBorderColor(IndexedColors.SKY_BLUE.getIndex());
        cellStyle.setBorderLeft(BorderStyle.THIN);
        //左边框
        cellStyle.setLeftBorderColor(IndexedColors.SKY_BLUE.getIndex());
        cellStyle.setBorderTop(BorderStyle.THIN);
        //上边框
        cellStyle.setRightBorderColor(IndexedColors.SKY_BLUE.getIndex());
        cellStyle.setBorderRight(BorderStyle.THIN);
        //右边框
        cellStyle.setTopBorderColor(IndexedColors.SKY_BLUE.getIndex());
        // 自动换行
        cellStyle.setWrapText(true);
        return cellStyle;
    }


    // 部门主键不为空时，日期格式只有某月。为空时，日期格式为某年统计的第一个月为某年某月，其余为某月
    public void conversionDateFormat(List<ProjectStatisticsByMonth> projectStatisticsByMonths, Integer deptId) {
        for (ProjectStatisticsByMonth projectStatisticsByMonth : projectStatisticsByMonths) {
            List<ProjectStatisticsByMonth.StatisticsDateAndWorkTime> statisticsDateAndWorkTimes = projectStatisticsByMonth.getStatisticsDateAndWorkTimes();
            // 部门主键不为空时，日期格式只有某月
            if (deptId != null) {
                for (ProjectStatisticsByMonth.StatisticsDateAndWorkTime statisticsDateAndWorkTime : statisticsDateAndWorkTimes) {
                    String date = statisticsDateAndWorkTime.getDate();
                    StringBuilder stringBuilder = new StringBuilder(date);
                    // 删除年份
                    stringBuilder.delete(0, 5);
                    statisticsDateAndWorkTime.setDate(stringBuilder.toString());
                }
            } else {
                for (int i = statisticsDateAndWorkTimes.size() - 1; i > 0; i--) {
                    String date1 = statisticsDateAndWorkTimes.get(i).getDate();
                    String date2 = statisticsDateAndWorkTimes.get(i - 1).getDate();
                    // 如果不为当年统计数据的第一个月，则删除年份
                    if (date1.charAt(2) == date2.charAt(2) && date1.charAt(3) == date2.charAt(3)) {
                        StringBuilder dateBuilder1 = new StringBuilder(date1);
                        // 删除年份
                        dateBuilder1.delete(0, 5);
                        statisticsDateAndWorkTimes.get(i).setDate(dateBuilder1.toString());
                    }
                }
            }
        }
    }

    // 获取用户权限
    public List<String> getUserAuthority(Integer userId) {
        // 获取用户角色
        List<Integer> role = UserRoleLevelUtils.getRole(userId);
        // 获取角色对应权限id
        List<Integer> list = UserRoleLevelUtils.getlevelIds(role);
        // 获取权限对象
        List<WorkLevel> workLevels = UserRoleLevelUtils.getlevelByIds(list);
        if (CollectionUtil.isEmpty(workLevels)) {
            return null;
        }
        // 获取权限名
        List<String> collect = workLevels.stream().map(WorkLevel::getName).collect(Collectors.toList());
        return collect;
    }

    // 获取用户当前权限对应的级别
    public List<Integer> getUserRank(List<WorkLevel> workLevels, String levelName) {
        List<Integer> rank = new ArrayList<>();
        for (WorkLevel workLevel : workLevels) {
            if (workLevel.getName().equals(levelName)) {
                rank.add(workLevel.getRank());
            }
        }
        return rank;
    }

}
